/* -*- mode: c++; c-basic-offset: 3; -*- */
#include "html/Attrib.hh"
#include "html/writer.hh"

using namespace std;

//======================================  Construct a null attribute
html::attrib::attrib(void)
{}

//======================================  Copy an attribute
html::attrib::attrib(const attrib& x) 
{}

//======================================  Destroy an attribute
html::attrib::~attrib(void) 
{}

//======================================  set up for writing
void
html::attrib::setWrite(writer& w) const {
}

//======================================  Test for default attribute
bool 
html::attrib::hasArg(void) const {
    return true;
}

//======================================
bool 
html::attrib::isDefault(void) const {
    return false;
}

//======================================  Describe the attribute
void 
html::attrib::putAttr(writer& w) const {
   w.text(std::string("\"") + valString() + "\"");
}

//======================================  Describe the attribute
string
html::attrib::valString(void) const {
   return "";
}

//==========================================================================
//
//              Attribute list class
//
//==========================================================================

//======================================  Construct an empty attribute list
html::attList::attList(void) {
}

//======================================  Attribute list destructor
html::attList::~attList(void) {
    clear();
}

//======================================  Add a specified attribute.
html::attrib& 
html::attList::addAttr(const std::string& name, const attrib& a) {
   mList[name].reset(a.clone());
   return *(mList.find(name)->second);
}

//======================================  Append to a defined attribute.
html::attrib& 
html::attList::appendAttr(const std::string& name, const attrib& a, int sep) {
    auto p = mList.find(name);
    if (p == mList.end() || (p->second)->isNull()) return addAttr(name, a);
    (p->second)->append(a, sep);
    return *(p->second);
}

//======================================  Replace a specified attribute.
html::attrib& 
html::attList::repAttr(const std::string& name, const attrib& a) {
    attr_iter p = mList.find(name);
    if (p == mList.end()) return addAttr(name, a);
    p->second.reset(a.clone());
    return *(p->second);
}

//======================================  Clear the attribute list
void
html::attList::clear(void) {
    mList.clear();
}

//======================================  Test if attribute exists
bool
html::attList::exists(const std::string& name) const {
    return mList.find(name) != mList.end();
}

//======================================  Test if attribute exists
const html::attrib&
html::attList::find(const std::string& name) const {
    return *(mList.find(name)->second);
}

//======================================  Remove an attribute
bool
html::attList::isDefault(void) const {
    for (const_attr_iter i=mList.begin() ; i != mList.end() ; i++) {
        if (!i->second->isDefault()) return false;
    }
    return true;
}

//======================================  Remove an attribute
void
html::attList::merge(const attList& x) {
    for (const_attr_iter i=x.mList.begin() ; i != x.mList.end() ; i++) {
        attr_iter j = mList.find(i->first);
	if (j == mList.end()) {
	    addAttr(i->first, *(i->second));
	} else if (j->second->isDefault()) {
	    mList.erase(j);
	    addAttr(i->first, *(i->second));
	}
    }
}

//======================================  Remove an attribute
void 
html::attList::remAttr(const std::string& name) {
    mList.erase(name);
}

//======================================  Put tag string to html writer
void 
html::attList::putAttr(writer& w) const {
    for (const_attr_iter i=mList.begin() ; i != mList.end() ; i++) {
        if (!(i->second)->isDefault()) {
	    w.text(" ");
	    string txt = i->first;
	    if ((i->second)->hasArg()) {
	        txt += "=\"";
		txt += (i->second)->valString() + "\"";
	    }
	    w.text(txt);
	}
    }
}

//======================================  Set up writer for further processing
void 
html::attList::setWrite(writer& w) const {
}
