/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "html/color.hh"
#include "html/font.hh"
#include "html/genAttr.hh"
#include "html/size.hh"
#include "html/style.hh"
#include "html/textAttr.hh"
#include "html/writer.hh"

using namespace html;
using namespace std;

//======================================  Default Constructor
styleAttrib::styleAttrib(void) {}

//======================================  Default Constructor
styleAttrib::styleAttrib(const string& att)
    : mStyle(att)
{}

//======================================  Default Constructor
styleAttrib::styleAttrib(const textAttr& attrs) {
    mStyle = attrs.getTextStyle();
}

//======================================  Default Constructor
styleAttrib*
styleAttrib::clone(void) const {
    return new styleAttrib(*this);
}

//======================================  Add an attribute to the style string
void
styleAttrib::addAttr(const std::string& name, const attrib& att) {
    mStyle += name + ":" + att.valString() + "; ";
}

//======================================  Add an attribute to the style string
void
styleAttrib::append(const attrib& att, int sep) {
    mStyle += sep;
    mStyle += att.valString();
}

//======================================  Set the color attribute
styleAttrib& 
styleAttrib::setColor(const color& c) {
    addAttr("color", c);
    return *this;
}

//======================================  Set the font face
styleAttrib& 
styleAttrib::setFace(const font& f) {
    addAttr("font-family", f);
    return *this;
}

//======================================  Set the font size
styleAttrib& 
styleAttrib::setSize(const size& sz) {
    addAttr("font-size", sz);
    return *this;
}

//======================================  set the font weight
styleAttrib& 
styleAttrib::setWeight(const std::string& c) {
    addAttr("font-weight", genAttr(c));
    return *this;
}

//======================================  return the value string.
std::string
styleAttrib::valString(void) const {
    return mStyle;
}

//======================================  Default Constructor
StyleClass::StyleClass(const std::string& name, const styleAttrib& att)
    : class_name(name), style_attr(att)
{}

//======================================  clone method
StyleClass*
StyleClass::clone(void) const {
    return new StyleClass(*this);
}

//======================================  Write out attribute list
void 
StyleClass::write(writer& w) const {
    w.text(class_name + " {" + style_attr.valString() + "}");
}

//======================================  Default Constructor
style::style(void) {
}


//======================================  Default Constructor
style*
style::clone(void) const {
    return new style(*this);
}

//======================================  Add a style class to the dictionary
void 
style::addClass(const StyleClass& c) {
    addObject(c);
}
